@@ -2,6 +2,7 @@
 
 import inspect
 import re
+import textwrap
 import warnings
 from contextlib import suppress
 from functools import partial
@@ -10,6 +11,7 @@ from typing import (
     Any,
     Callable,
     ClassVar,
+    Collection,
     Dict,
     Final,
     List,
@@ -171,85 +173,85 @@ def load(__call: KwargsCallable[_T], __get: Callable[[str], Any], **overrides) -
     return __call(**overrides)
 
 
-# def _get_doc(target, description, **attribute_descriptions):
-#     if isinstance(getattr(target, "__doc__", None), str) and getattr(target, "__doc__").strip():
-#         parsed = docs.parse(getattr(target, "__doc__"))
-#         if description is None:
-#             description = (parsed.short_description or "").strip()
-#             if parsed.long_description:
-#                 description = f"{description}\n\n{parsed.long_description.strip()}"
-#         for param in parsed.params:
-#             attribute_descriptions.setdefault(param.arg_name, param.description)  # type: ignore
-#
-#     parts = [f"{textwrap.dedent((description or '').rstrip())}\n"]
-#     if attribute_descriptions:
-#         parts.append("Attributes:")
-#     for k, v in attribute_descriptions.items():
-#         parts.append(f"    {k}:\n{textwrap.indent(textwrap.dedent(v).rstrip(), ' ' * 8)}")
-#     return "\n".join(parts).rstrip()
-#
-#
-# def _get_class_namespace(cls, target, skip_parameters, raise_error) -> dict:
-#     if target is None:
-#         return {}
-#
-#     skip = f"while extending {cls.__module__}.{cls.__qualname__} for {target!r}, skipping parameter {{}}: {{}}"
-#
-#     existing: Dict[str, Any] = {cls.as_setting_name(k): ann for k, ann in cls.__annotations__.items()}
-#
-#     annotations: Dict[str, Any] = {}
-#     defaults: Dict[str, Any] = {}
-#
-#     for i, parameter in enumerate(inspect.signature(target).parameters.values()):
-#
-#         if i in skip_parameters or parameter.name in skip_parameters:
-#             continue
-#
-#         if parameter.kind in (inspect.Parameter.VAR_KEYWORD, inspect.Parameter.VAR_POSITIONAL):
-#             reason = skip.format(parameter.name, "variadic parameters not allowed")
-#             if raise_error:
-#                 raise TypeError(reason)
-#             warnings.warn(reason)
-#             continue
-#
-#         if parameter.annotation is inspect.Parameter.empty:
-#             reason = skip.format(parameter.name, "parameter missing annotation")
-#             if raise_error:
-#                 raise TypeError(reason)
-#             warnings.warn(reason)
-#             continue
-#
-#         name = cls.as_setting_name(parameter.name)
-#         if not cls.is_setting(name):
-#             reason = skip.format(parameter.name, f"the generated setting name {name} is invalid")
-#             if raise_error:
-#                 raise TypeError(reason)
-#             warnings.warn(reason)
-#             continue
-#
-#         if name in existing and existing[name] != parameter.annotation:
-#             raise TypeError(
-#                 skip.format(
-#                     parameter.name,
-#                     (
-#                         f"the annotation {inspect.formatannotation(parameter.annotation)} does not match "
-#                         f"old annotation {inspect.formatannotation(existing[name])}"
-#                     ),
-#                 ),
-#             )
-#
-#         if name in annotations:
-#             raise TypeError(skip.format(parameter.name, f"duplicate setting name {name}"))
-#
-#         annotations[name] = parameter.annotation
-#
-#         if parameter.default is not inspect.Parameter.empty:
-#             defaults[name] = parameter.default
-#
-#     return {
-#         "__annotations__": annotations,
-#         **defaults,
-#     }
+def _get_doc(target, description, **attribute_descriptions):
+    if isinstance(getattr(target, "__doc__", None), str) and getattr(target, "__doc__").strip():
+        parsed = docs.parse(getattr(target, "__doc__"))
+        if description is None:
+            description = (parsed.short_description or "").strip()
+            if parsed.long_description:
+                description = f"{description}\n\n{parsed.long_description.strip()}"
+        for param in parsed.params:
+            attribute_descriptions.setdefault(param.arg_name, param.description)  # type: ignore
+
+    parts = [f"{textwrap.dedent((description or '').rstrip())}\n"]
+    if attribute_descriptions:
+        parts.append("Attributes:")
+    for k, v in attribute_descriptions.items():
+        parts.append(f"    {k}:\n{textwrap.indent(textwrap.dedent(v).rstrip(), ' ' * 8)}")
+    return "\n".join(parts).rstrip()
+
+
+def _get_class_namespace(cls, target, skip_parameters, raise_error) -> dict:
+    if target is None:
+        return {}
+
+    skip = f"while extending {cls.__module__}.{cls.__qualname__} for {target!r}, skipping parameter {{}}: {{}}"
+
+    existing: Dict[str, Any] = {cls.as_setting_name(k): ann for k, ann in cls.__annotations__.items()}
+
+    annotations: Dict[str, Any] = {}
+    defaults: Dict[str, Any] = {}
+
+    for i, parameter in enumerate(inspect.signature(target).parameters.values()):
+
+        if i in skip_parameters or parameter.name in skip_parameters:
+            continue
+
+        if parameter.kind in (inspect.Parameter.VAR_KEYWORD, inspect.Parameter.VAR_POSITIONAL):
+            reason = skip.format(parameter.name, "variadic parameters not allowed")
+            if raise_error:
+                raise TypeError(reason)
+            warnings.warn(reason)
+            continue
+
+        if parameter.annotation is inspect.Parameter.empty:
+            reason = skip.format(parameter.name, "parameter missing annotation")
+            if raise_error:
+                raise TypeError(reason)
+            warnings.warn(reason)
+            continue
+
+        name = cls.as_setting_name(parameter.name)
+        if not cls.is_setting(name):
+            reason = skip.format(parameter.name, f"the generated setting name {name} is invalid")
+            if raise_error:
+                raise TypeError(reason)
+            warnings.warn(reason)
+            continue
+
+        if name in existing and existing[name] != parameter.annotation:
+            raise TypeError(
+                skip.format(
+                    parameter.name,
+                    (
+                        f"the annotation {inspect.formatannotation(parameter.annotation)} does not match "
+                        f"old annotation {inspect.formatannotation(existing[name])}"
+                    ),
+                ),
+            )
+
+        if name in annotations:
+            raise TypeError(skip.format(parameter.name, f"duplicate setting name {name}"))
+
+        annotations[name] = parameter.annotation
+
+        if parameter.default is not inspect.Parameter.empty:
+            defaults[name] = parameter.default
+
+    return {
+        "__annotations__": annotations,
+        **defaults,
+    }
 
 
 class Config:
@@ -275,30 +277,30 @@ class Config:
         cls: "_TConfig",
         *,
         namespace: str = "",
-        # target: Optional[Callable] = None,
-        # skip_parameters: Collection[Union[int, str]] = (),
-        # raise_error=False,
-        # description: str = None,
-        # **attribute_descriptions: str,
+        target: Optional[Callable] = None,
+        skip_parameters: Collection[Union[int, str]] = (),
+        raise_error=False,
+        description: str = None,
+        **attribute_descriptions: str,
     ) -> "_TConfig":
         """Returns a subclass update using given input.
 
         Args:
             namespace: new namespace for the class
-            # target: callable to get additional setting parameters from
-            # skip_parameters: names of target parameters to skip
-            # raise_error: whether to raise an error on invalid target parameters
-            # description: new class description; do not use with doc parameter
-            # **attribute_descriptions: new settings' descriptions; do not use with doc parameter
+            target: callable to get additional setting parameters from
+            skip_parameters: names of target parameters to skip
+            raise_error: whether to raise an error on invalid target parameters
+            description: new class description; do not use with doc parameter
+            **attribute_descriptions: new settings' descriptions; do not use with doc parameter
         """
 
-        # class_namespace = _get_class_namespace(cls, target, skip_parameters, raise_error)
-        # class_namespace["__doc__"] = _get_doc(target, description, **attribute_descriptions)
+        class_namespace = _get_class_namespace(cls, target, skip_parameters, raise_error)
+        class_namespace["__doc__"] = _get_doc(target, description, **attribute_descriptions)
 
         return type(  # type: ignore
             cls.__name__,
             (cls, ),
-            {},
+            class_namespace,
             namespace=namespace,
         )
 
@@ -337,6 +339,7 @@ class Config:
         self.__defaults = {}
         self.__env = __env
         self.__app = __app
+        self.__configured = False
 
         if namespace is not EMPTY_NAMESPACE:
             namespace = namespace or self.__class__.namespace
@@ -414,6 +417,9 @@ class Config:
         orig_name, setting_name = __name, self.as_setting_name(__name)
         if setting_name not in self.__attributes:
             return super().__getattribute__(orig_name)
+        if not self.__configured:
+            self.__configured = True
+            self.populate_from_env()
         if setting_name not in self.__dict__:
             setattr(self, setting_name, self._get(setting_name))
         return super().__getattribute__(setting_name)
